"""
Sia dato un testo che contiene un poema, ovvero una successione di versi in rima.
Il poema è contenuto in un file, un verso per riga.

Vogliamo analizzarlo per estrarne la struttura prosodica, ovvero lo schema poetico in esso usato.
Per far questo sono utili le seguenti definizioni:
    - un 'elemento sonoro' (ES) è una successione massimale di 1 o più consonanti seguite da 1 o più vocali
        - prima tutte le consonanti
        - poi tutte le vocali (aeiouyj)
        - ignorando eventuali caratteri non alfabetici come spazi, numeri e segni di interpunzione 
        - togliendo gli accenti dalle lettere accentate
        - e ignorando la differenza tra maiuscole e minuscole
        NOTA:   fa eccezione il primo ES di un verso, che può essere composto da sole vocali (vedi esempio in fondo)
                e l'ultimo ES, che può essere composto di sole consonanti
    - un verso è composto da una successione di elementi sonori, l'ultimo dei quali è chiamato 'finale'
        Esempio:      
        Se il verso è "Paperino andò al mare a pescare" 
            - gli elementi sonori sono     ["pa", "pe", "ri", "noa", "ndoa", "lma", "rea", "pe", "sca", "re"]
            - la finale è                   "re"
            - il verso è lungo              10 ES
        notate che le lettere accentate hanno perso l'accento e non ci interessa la distinzione tra maiuscole e minuscole
    - la struttura prosodica di una poesia è una lista di interi, uno per ciascun verso
    - per ciascun verso si considerano sia il numero di ES (#ES) che la sua finale
    - al primo verso va associato il numero 0
    - a ciascuno dei versi successivi va associata:
        - l'intero che è stato già associato ad un verso precedente che ha stesso #ES e finale
        - altrimenti un nuovo intero (che segue l'ultimo già usato)
    Esempio:
        se la poesia è quella qui sotto                     gli elementi sonori sono                                    #ES finale   prosodia
        '''
        Dì pestaggio tessessi allarmai, Partenopea!         di pe sta ggio te sse ssia lla rmai pa rte no pea           13  pea         0
        Sembrò svieremo imbarcate, aumentarono usurpai?     se mbro svie re moi mba rca teau me nta ro nou su rpai      14  rpai        1
        Flash privé spirereste? Pentecoste deturpai         fla shpri ve spi re re ste pe nte co ste de tu rpai         14  rpai        1
        scrost, direttamante arrischiai,                    scro stdi re tta ma ntea rri schiai                          8  schiai      2
        odi attuazione vernicera Partenopea.                o dia ttua zio ne ve rni ce ra pa rte no pea                13  pea         0
        Psion trentacinque preesistiti calzascarpe          psio ntre nta ci nque pree si sti ti ca lza sca rpe         13  rpe         3
        nobilt fiacchi vedesti avvertirsi spermatozoi?      no bi ltfia cchi ve de stia vve rti rsi spe rma to zoi      14  zoi         4
        Igloo rubi incassando giurati spermatozoi!          i gloo ru bii nca ssa ndo giu ra ti spe rma to zoi          14  zoi         4
        Saprai reputasse inebriai                           sa prai re pu ta ssei ne briai                               8  briai       5
        man l'ballaste segnaleremo soprascarpe.             ma nlba lla ste se gna le re mo so pra sca rpe              13  rpe         3
        '''
        la struttura prosodica è     [0, 1, 1, 2, 0, 3, 4, 4, 5, 3]
        Che è formata da due sequenze uguali di 5 elementi che seguono lo schema [0, 1, 1, 2, 0]
            infatti [0, 1, 1, 2, 0] è equivalente a [3, 4, 4, 5, 3]
        l'elenco dei numeri di ES è  [13,    14,     14,     8,        13,    13,    14,    14,    8,       13   ]
        l'elenco delle finali è      ['pea', 'rpai', 'rpai', 'schiai', 'pea', 'rpe', 'zoi', 'zoi', 'briai', 'rpe']

        La funzione deve tornare la tupla di 4 valori: (prosodia, periodo, lunghezze, finali), ovvero per questo esempio
          ( [0, 1, 1, 2, 0, 3, 4, 4, 5, 3], 5, [13, 14, 14, 8, 13, 13, 14, 14, 8, 13 ], 
            ['pea', 'rpai', 'rpai', 'schiai', 'pea', 'rpe', 'zoi', 'zoi', 'briai', 'rpe'])
"""

def ex1(poetry_filename):
    # insert here your code
    pass
    with open(poetry_filename, encoding='utf8') as F:
        versi = F.readlines()
    return process(versi)

def process(versi):
    puliti = [pulisci(verso) for verso in versi ]
    #print(*puliti, sep='\n')
    elementi_sonori = [ sillabe(verso) for verso in puliti ]
    #print(*elementi_sonori, sep='\n')
    finali = [ ES[-1] for ES in elementi_sonori ]
    lunghezze = [ len(ES) for ES in elementi_sonori ]
    coppie = list(zip(finali, lunghezze))
    rime = { coppie[0] : 0 }
    prosodia = [0]
    last = 0
    for coppia in coppie[1:]:
        if coppia not in rime:
            last += 1
            rime[coppia] = last
        prosodia.append(rime[coppia])
    #prosodia = ''.join(prosodia)
    #print('end', finali)
    #print('len', lunghezze)
    #print('pro', prosodia)
    modulo = trova_modulo(prosodia)
    return prosodia, modulo, lunghezze, finali

def trova_modulo(prosodia):
    N = len(prosodia)
    for size in range(3, N+1):
        if N%size != 0: continue
        modulo = prosodia[:size]
        lettere  = { i : modulo[i] for i in range(size) }
        for i in range(size, len(prosodia), size): # FIXME +1?
            snip = prosodia[i:i+size]
            #print(size, modulo, snip, lettere)
            if not corrisponde(lettere, snip):
                #print("non corrisponde", modulo, snip)
                break
        else:
            return size
    return N


def corrisponde(lettere, pezzo):
    mappa_lp = {}
    mappa_pl = {}
    for i,c in enumerate(pezzo):
        C = lettere[i]
        if c not in mappa_lp:
            mappa_lp[c] = C
        if C not in mappa_pl:
            mappa_pl[C] = c
        if mappa_lp[c] != C:
            #print(mappa_lp[c], '<>', C)
            return False
        if mappa_pl[C] != c:
            #print(mappa_pl[C], '<>', c)
            return False
    else:
        #print(mappa_lp, mappa_pl)
        return True

def sillabe(verso):
    #print(verso)
    ES = ''
    sillabe = []
    vocali = 'aeioujy'
    for c in verso:
        if c in vocali:
            ES += c
        else:
            if not ES:
                ES = c
            elif ES[-1] in vocali:
                sillabe.append(ES)
                ES = c
            else:
                ES += c
    if ES:
        sillabe.append(ES)
    #print(sillabe)
    return sillabe

interpun  = ".,':;!?\n "
accentate = 'àáâãäåèéêëìíîïòóôõöøùúûüýÿ'
piane     = 'aaaaaaeeeeiiiioooooouuuuyy'
tran      = str.maketrans(accentate, piane, interpun)

def pulisci(verso):
    verso = verso.lower()
    pulito = verso.translate(tran)
    #pulito = [ c for c in verso if c.isalpha() ]
    #pulito = ''.join(pulito)
    #print(pulito)
    return pulito

def ultime_vocali(verso):
    vocali = ''
    i = -1
    while len(vocali) < 2:
        c = verso[i]
        if c in 'aeiou':
            vocali = c + vocali
        i -= 1
    return vocali

if __name__ == '__main__':
    import sys
    # insert here your personal tests
    pass
    if len(sys.argv) > 1:
        print(*ex1(sys.argv[1]), sep='\n')

